program DSI_simple;
{

Introduction

This Pascal/Delphi code is a simple presentation of a simulation series of group
behavior process in the DSI / NSL paradigm - original article is:

Nowak, A., Szamrej, J., & Latan, B. (1990). From private attitude to public opinion: A dynamic theory of social impact. Psychological Review, 97, 362-376. doi:10.1037/0033-295X.97.3.362

and our article

Rak, T., Kulesza, W., & Chrobot, N. (2018). The Internet Chess Rules Change: Queen Equals to Pawn. To Spread the Concept You Do Not Need a Queen But a Lot of Pawns. Social Psychological Bulletin, in print

The code complies with detailed guidelines from the original article and our article
introducing the element of global communication into the code.

All optimization of code are omitted, in order to show the simplest way to create
simulations and obtain similar as articles results.

Nevertheless - for the needs of our project a very extensive simulator was
created in a graphical mode with a lot of options to determine, for example, the plane,
various types of calculations of impact forces, report generation and others.
The simulator was created as part of the interdisciplinary PIAUZ.PL project and
can be available through SOCIALMATH.PL company managing the simulator and project.

PIAUZ is a project of creating a complex simulator ready to work and a
graphical presentation of simulation run for students and researchers. All
persons interested in participating in the project and its extension
are encouraged to contact us.

NOTE: The code will be more understandable for one familiar with the article. ;)

NOTE2: obtaining identical results depends on such factors as the time and size
of the simulation, space (plane vs torus), and (primarily) the social distance;
we encourage you to test different boundary conditions or even different ways
of calculating the distance - the results can be very interesting
}


{$APPTYPE CONSOLE}
uses
  SysUtils;

// Constants determining boundary conditions of the simulation can of course be
// changed; by default those from the original article have been determined.

const simulation_size = 40; // Simulation matrix 40x40 individuals
      loops = 100; // DSI = 10
      range_limit = 10; // original article contains increase of efficiency of
      // simulation, forcing range of distance limit, see pp. 368; one can of
      // of course, increase this limit to the size of the simulation because
      // modern personal computers are much more efficient
      GCM = True; // orginal model it does not contain global communication and
      // this tag, if set to "false", will perform calculations without this
      // communication. To get results for global communication consistent with
      // our article one should first get start and final proportions
      // with last procedure, and then change this field to "true" and re-run


      // person in the matrix is represented by these three fields; their
      // understanding is described on page 365 in the original article
type Individual = record
               Attitude, Supportiveness, Persuasiveness : byte;
             end;
var
  Matrix : array [1..simulation_size, 1..simulation_size] of Individual;
  NoChanges, MaxNoChanges: Integer;
  // Any simulation should have also end ;) so below is a numerical condition
  // for counting max steps with no-change in matrix;

function EuclideanDistance(x1, y1, x2, y2 : Real): Real;
{
a description of social distance can be found on page 366 in "immediacy" section

note 1: of course, one can increase efficiency code by, for example, ommiting
power of result and then square root of it; or also use tables and reading already
calculated distance values; nevertheless, we wanted to show the simplest method
of reproducing the results of the original article (NSL / DSI).

note 2: the original article contained information
"To avoid dividing by 0, the distance on each axis was increased by 1."
We know from personal information from the authors that it was done more
or less like this - with ading +1 to each axis.
Of course, one can also use an ordinary Euclidean distance, remember only to avoid
auto-influence and division by zero, but in effect the simulation will give
slightly different results for proportion at end; reason: the distance is the most
important element of influence.
}
begin
  Result := Sqrt(Sqr(Abs(x1 - x2) + 1) + (Sqr(Abs(y1 - y2))+ 1));
//  Result := Sqrt(Sqr(Abs(x1 - x2)) + (Sqr(Abs(y1 - y2))));
end;

function GiveProportion: Real;
{
At each stage of the simulation, one can check the current state of the calculation,
for example the matrix view, or the proportion of people representing each view;
this is a simple function that counts people with the "1" view...
}

var i,j,k: Integer;
begin
  k := 0;
  for i:= 1 to simulation_size do
    for j:= 1 to simulation_size do
      if Matrix[i,j].Attitude = 1
        then inc(k);
  Result := (k / Sqr(simulation_size))*100;
end;

function GiveN: Integer;
{
and here's simple function counting number of people with "1" view
}
var i,j,k: Integer;
begin
  k := 0;
  for i:= 1 to simulation_size do
    for j:= 1 to simulation_size do
      if Matrix[i,j].Attitude = 1
        then inc(k);
  Result := k;
end;

procedure ShowMatrix;
{
here one can print simulation matrix in classic way - similar to this from
original article; it can be helpfull in testing of algoritm
}
var i,j: Integer;
begin
  for i:= 1 to simulation_size do
    begin
      for j:= 1 to simulation_size do
         if Matrix[i,j].Attitude = 0
           then Write('-')
           else Write('I');
      WriteLn;
    end;
end;

procedure RandomMatrix(start_proportion: Integer);
{
on page 367 in the original article one can find a way to create a random matrix
- random characteristics are assigned to individuals in matrix, and a fixed percentage
- proportion - receives the starting view of "1"; it is a way of representing
a random society with "proportion" of people of a different opinion and allows
for example to test the survival of a minority group

0 and 1 represents oposite points of view
}
var i, j, k : Integer;
begin
  Randomize;
  for i:= 1 to simulation_size do
    for j:= 1 to simulation_size do
      begin
        Matrix[i,j].Attitude := 0;
        Matrix[i,j].Supportiveness := Random(101);
        Matrix[i,j].Persuasiveness := Random(101);
      end;
  k := 0;
  repeat
    i := Random(simulation_size)+1;
    j := Random(simulation_size)+1;
    if Matrix[i,j].Attitude = 0
      then begin
             Matrix[i,j].Attitude := 1;
             Inc(k);
      end;
  until k >= ((start_proportion /100) * Sqr(simulation_size));
  NoChanges := 0;
end;

procedure Immediacy;
{
in original article in "The Basic Formulas and Rules of Simulation" one can find
rules of change and formulas for counting Supportiveness and Persuasiveness;
here's simple way of counting 'em and using The Change Rule;

the simulation can be carried out simultaneously for all people in the matrix
or in Monte Carlo mode, i.e. selecting random individual units in the matrix
and applying change formula for it. The results are usually convergent, with
the difference that the Monte Carlo method gives slightly less extreme results
for small groups. The Monte Carlo method has been presented, because it
is simpler in code and more understandable.
}


var SumOfSupportiveness, SumOfPersuasiveness: Real; // formulas are on 366 page
    SumOfGlobalSupportiveness, SumOfGlobalPersuasiveness: Real;
    SupportN,PersuasivenessN: Integer; // number of individuals representing same
    // and oposite point of view
    i,j,k,l: Integer;
begin
  SumOfSupportiveness := 0; // variable initialize
  SumOfPersuasiveness := 0;
  SumOfGlobalSupportiveness := 0;
  SumOfGlobalPersuasiveness := 0;
  SupportN := 0;
  PersuasivenessN := 0;

  Randomize; // randomizer initialize
  i := Random(simulation_size)+1; // selecting individual
  j := Random(simulation_size)+1; // influenced

  for k := 1 to simulation_size do
    for l := 1 to simulation_size do
      if ((Abs(i-k) <= range_limit)) and ((Abs(j-l) <= range_limit)) then
      // counting only in range limit
      if Matrix[k,l].Attitude = Matrix[i,j].Attitude
        then Inc(SupportN)
        else Inc(PersuasivenessN); // counting number of individuals

  if SupportN = 0 then SupportN := 1;
  if PersuasivenessN = 0 then PersuasivenessN := 1;
  // this is to avoid division by 0 in some simulations and it does not matter too much ;)

  // calculating sum of Supportiveness and Persuasiveness from social matrix
  // explanation on page 366 in original article
  for k := 1 to simulation_size do
    for l := 1 to simulation_size do
      if ((Abs(i-k) <= range_limit)) and ((Abs(j-l) <= range_limit)) then
//      if (i <> k) and (j <> l) then // to avoid auto-influence - one can experiment with this setting
      if Matrix[k,l].Attitude = Matrix[i,j].Attitude
        then SumOfSupportiveness := SumOfSupportiveness +
        (Matrix[k,l].Supportiveness / Sqr(EuclideanDistance(i,j,k,l)) / SupportN)
        else SumOfPersuasiveness := SumOfPersuasiveness +
        (Matrix[k,l].Persuasiveness / Sqr(EuclideanDistance(i,j,k,l)) / PersuasivenessN);

   SumOfSupportiveness := Sqrt (SupportN) * ((SumOfSupportiveness));
   SumOfPersuasiveness := Sqrt (PersuasivenessN) * ((SumOfPersuasiveness));
   // scaling impact by number of individuals; in further articles Nowak and et al
   // also experimented with the omission of such scaling; we encourage to experiment;


   // if GCM is on - as in our article - the impact forces in the global network
   // are also calculated, and then, according to our proposal, added to the
   // impact of the classical "analog" social network
   // our formula proposition is in "online persuasiveness and online supportiveness"
   // section in our article
   if GCM
     then begin
            for k := 1 to simulation_size do
              for l := 1 to simulation_size do
                if ((Abs(i-k) <= range_limit)) and ((Abs(j-l) <= range_limit)) then
          //      if (i <> k) and (j <> l) then
                if Matrix[k,l].Attitude = Matrix[i,j].Attitude
                  then SumOfGlobalSupportiveness := SumOfGlobalSupportiveness +
                  (Matrix[k,l].Supportiveness  / SupportN)
                  else SumOfGlobalPersuasiveness := SumOfGlobalPersuasiveness +
                  (Matrix[k,l].Persuasiveness  / PersuasivenessN);
            // Naturally, you can also omit the declaration of these two variables
            // and work only on variables corresponding to ordinary communication,
            // but so user can control the values of these calculations - if he wants
            SumOfGlobalSupportiveness := Sqrt (SupportN) * ((SumOfGlobalSupportiveness));
            SumOfGlobalPersuasiveness := Sqrt (PersuasivenessN) * ((SumOfGlobalPersuasiveness));
            SumOfPersuasiveness := SumOfPersuasiveness + SumOfGlobalPersuasiveness;
            SumOfSupportiveness := SumOfSupportiveness + SumOfGlobalSupportiveness;
     end;

   // using The Change Rule - if Persuasiveness of social matrix is stronger than
   // Supportiveness then indiviudal is forced to change its view and its characteristics
   // are assigned randomly with proposition of Nowak et al
   if SumOfPersuasiveness > SumOfSupportiveness
     then begin
            NoChanges := 0;
            if Matrix[i,j].Attitude = 0
              then Matrix[i,j].Attitude := 1
              else Matrix[i,j].Attitude := 0;
            Matrix[i,j].Supportiveness := Random(101);
            Matrix[i,j].Persuasiveness := Random(101);
     end else Inc(NoChanges); // counting no-changes in run, to get simulation-end condition
end;

procedure LoopOfRuns(proportion: Integer);
{
main idea in presenting results - naturally, besides tracking the social network
view - is generating a series of simulations and calculating the average ratios
of final proportions in relation to final proportion of group representing
the view of "1", like on Figure 5 in original article; here's simple procedure
for generating N="loops" of single runs, and print average results...
}
var MeanProportionOnEnd: Real;
    i: Integer;
begin
  MeanProportionOnEnd := 0;
  for i := 1 to loops do
  begin
    RandomMatrix(proportion);
    repeat
      Immediacy;
    until NoChanges > MaxNoChanges;
    MeanProportionOnEnd := MeanProportionOnEnd + GiveProportion;
  end;
  MeanProportionOnEnd := MeanProportionOnEnd / loops;
  WriteLn('; Proportion end (mean): ',MeanProportionOnEnd:0:3);
end;

procedure LoopOfLoops;
{
... and here's procedure that makes series of series for initial proportions in
10-90% range as on figure 5 in original article
}
var i: Integer;
begin
  for i := 1 to 9 do
  begin
    Write('Proportion start: ',i*10);
    LoopOfRuns(i*10);
//    ShowMatrix;  // You can watch end-matrix of last simulation... see for yourself
                   // how often "typical" results from original article appear.
  end;

end;

begin
  // Maximum number of iterations with no changes can be counted in various way -
  // heres just an sample interpretation of orginal article instruction:
  // "Steps of the simulation continued until there wereno further changes in attitude
  // in successive runs".
  // Note: this may be an interesting experiment for reader because depending on different
  // boundary conditions of the simulation, a longer simulation time does or doesn't
  // favor small groups. We call it the Mechanics of the Revolution.

  MaxNoChanges := ((simulation_size * simulation_size) div Round(Sqrt(simulation_size) * 0.5))*2;

  LoopOfLoops;

  ReadLn;
  // Obtained results should be in line with figure 5 in original article and
  // figure 2 from our article
end.


